@@ -13,6 +13,7 @@ use std::str;
 #[cfg(unix)]
 use std::sync::Mutex;
 use std::sync::OnceLock;
+use std::time::SystemTime;
 
 use chrono::prelude::*;
 
@@ -98,7 +99,7 @@ pub struct File<'dir> {
     /// instead.
     pub deref_links: bool,
 
-    /// The recursive directory size when total_size is used.
+    /// The recursive directory size when `total_size` is used.
     recursive_size: RecursiveSize,
 
     /// The extended attributes of this file.
@@ -135,6 +136,8 @@ impl<'dir> File<'dir> {
             RecursiveSize::None
         };
 
+        debug!("deref_links {}", deref_links);
+
         let mut file = File {
             name,
             ext,
@@ -343,7 +346,19 @@ impl<'dir> File<'dir> {
     /// Determine the full path resolving all symbolic links on demand.
     pub fn absolute_path(&self) -> Option<&PathBuf> {
         self.absolute_path
-            .get_or_init(|| std::fs::canonicalize(&self.path).ok())
+            .get_or_init(|| {
+                if self.is_link() && self.link_target().is_broken() {
+                    // workaround for broken symlinks to get absolute path for parent and then
+                    // append name of file; std::fs::canonicalize requires all path components
+                    // (including the last one) to exist
+                    self.path
+                        .parent()
+                        .and_then(|parent| std::fs::canonicalize(parent).ok())
+                        .map(|p| p.join(self.name.clone()))
+                } else {
+                    std::fs::canonicalize(&self.path).ok()
+                }
+            })
             .as_ref()
     }
 
@@ -702,6 +717,20 @@ impl<'dir> File<'dir> {
         }
     }
 
+    /// Converts a `SystemTime` to a `NaiveDateTime` without panicking.
+    ///
+    /// Fixes #655 and #667 in `Self::modified_time`, `Self::accessed_time` and
+    /// `Self::created_time`.
+    fn systemtime_to_naivedatetime(st: SystemTime) -> Option<NaiveDateTime> {
+        let duration = st.duration_since(SystemTime::UNIX_EPOCH).ok()?;
+
+        // FIXME: NaiveDateTime::from_timestamp_opt is deprecated since chrono 0.4.35
+        NaiveDateTime::from_timestamp_opt(
+            duration.as_secs().try_into().ok()?,
+            (duration.as_nanos() % 1_000_000_000).try_into().ok()?,
+        )
+    }
+
     /// This file’s last modified timestamp, if available on this platform.
     pub fn modified_time(&self) -> Option<NaiveDateTime> {
         if self.is_link() && self.deref_links {
@@ -712,8 +741,8 @@ impl<'dir> File<'dir> {
         }
         self.metadata
             .modified()
-            .map(|st| DateTime::<Utc>::from(st).naive_utc())
             .ok()
+            .and_then(Self::systemtime_to_naivedatetime)
     }
 
     /// This file’s last changed timestamp, if available on this platform.
@@ -743,8 +772,8 @@ impl<'dir> File<'dir> {
         }
         self.metadata
             .accessed()
-            .map(|st| DateTime::<Utc>::from(st).naive_utc())
             .ok()
+            .and_then(Self::systemtime_to_naivedatetime)
     }
 
     /// This file’s created timestamp, if available on this platform.
@@ -755,10 +784,10 @@ impl<'dir> File<'dir> {
                 _ => None,
             };
         }
-        match self.metadata.created() {
-            Ok(btime) => Some(DateTime::<Utc>::from(btime).naive_utc()),
-            Err(_) => None,
-        }
+        self.metadata
+            .created()
+            .ok()
+            .and_then(Self::systemtime_to_naivedatetime)
     }
 
     /// This file’s ‘type’.
