@@ -1,15 +1,16 @@
 use std::fmt::Debug;
 use std::path::Path;
 
-use ansiterm::{ANSIString, Style};
+use nu_ansi_term::{AnsiString as ANSIString, Style};
+use path_clean;
 use unicode_width::UnicodeWidthStr;
 
-use crate::fs::mounts::MountedFs;
 use crate::fs::{File, FileTarget};
 use crate::output::cell::TextCellContents;
 use crate::output::escape;
 use crate::output::icons::{icon_for_file, iconify_style};
 use crate::output::render::FiletypeColours;
+use crate::theme::FileNameStyle;
 
 /// Basically a file name factory.
 #[derive(Debug, Copy, Clone)]
@@ -26,6 +27,9 @@ pub struct Options {
     /// Whether to make file names hyperlinks.
     pub embed_hyperlinks: EmbedHyperlinks,
 
+    /// Whether to display files with their absolute path.
+    pub absolute: Absolute,
+
     /// Whether we are in a console or redirecting the output
     pub is_a_tty: bool,
 }
@@ -49,7 +53,6 @@ impl Options {
                 None
             },
             mount_style: MountStyle::JustDirectoryNames,
-            mounted_fs: file.mount_point_info(),
         }
     }
 }
@@ -69,9 +72,10 @@ enum LinkStyle {
 }
 
 /// Whether to append file class characters to the file names.
-#[derive(PartialEq, Eq, Debug, Copy, Clone)]
+#[derive(PartialEq, Eq, Debug, Default, Copy, Clone)]
 pub enum Classify {
     /// Just display the file names, without any characters.
+    #[default]
     JustFilenames,
 
     /// Always add a character after the file name depending on what class of
@@ -82,12 +86,6 @@ pub enum Classify {
     AutomaticAddFileIndicators,
 }
 
-impl Default for Classify {
-    fn default() -> Self {
-        Self::JustFilenames
-    }
-}
-
 /// When displaying a directory name, there needs to be some way to handle
 /// mount details, depending on how long the resulting Cell can be.
 #[derive(PartialEq, Debug, Copy, Clone)]
@@ -121,6 +119,14 @@ pub enum EmbedHyperlinks {
     On,
 }
 
+/// Whether to show absolute paths
+#[derive(PartialEq, Eq, Debug, Copy, Clone)]
+pub enum Absolute {
+    On,
+    Follow,
+    Off,
+}
+
 /// Whether or not to wrap file names with spaces in quotes.
 #[derive(PartialEq, Debug, Copy, Clone)]
 pub enum QuoteStyle {
@@ -149,9 +155,6 @@ pub struct FileName<'a, 'dir, C> {
 
     pub options: Options,
 
-    /// The filesystem details for a mounted filesystem.
-    mounted_fs: Option<&'a MountedFs>,
-
     /// How to handle displaying a mounted filesystem.
     mount_style: MountStyle,
 }
@@ -160,7 +163,9 @@ impl<'a, 'dir, C> FileName<'a, 'dir, C> {
     /// Sets the flag on this file name to display link targets with an
     /// arrow followed by their path.
     pub fn with_link_paths(mut self) -> Self {
-        self.link_style = LinkStyle::FullLinkPaths;
+        if !self.file.deref_links {
+            self.link_style = LinkStyle::FullLinkPaths;
+        }
         self
     }
 
@@ -185,6 +190,11 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
     /// width calculated.
     pub fn paint(&self) -> TextCellContents {
         let mut bits = Vec::new();
+        let (icon_override, filename_style_override) = match self.colours.style_override(self.file)
+        {
+            Some(FileNameStyle { icon, filename }) => (icon, filename),
+            None => (None, None),
+        };
 
         let spaces_count_opt = match self.options.show_icons {
             ShowIcons::Always(spaces_count) => Some(spaces_count),
@@ -199,9 +209,25 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
         };
 
         if let Some(spaces_count) = spaces_count_opt {
-            let style = iconify_style(self.style());
-            let file_icon = icon_for_file(self.file).to_string();
-            bits.push(style.paint(file_icon));
+            let (style, icon) = match icon_override {
+                Some(icon_override) => (
+                    if let Some(style_override) = icon_override.style {
+                        style_override
+                    } else {
+                        iconify_style(self.style())
+                    },
+                    icon_override
+                        .glyph
+                        .unwrap_or_else(|| icon_for_file(self.file))
+                        .to_string(),
+                ),
+                None => (
+                    iconify_style(self.style()),
+                    icon_for_file(self.file).to_string(),
+                ),
+            };
+
+            bits.push(style.paint(icon));
             bits.push(style.paint(" ".repeat(spaces_count as usize)));
         }
 
@@ -218,7 +244,7 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
             // indicate this fact. But when showing targets, we can just
             // colour the path instead (see below), and leave the broken
             // link’s filename as the link colour.
-            for bit in self.escaped_file_name() {
+            for bit in self.escaped_file_name(filename_style_override) {
                 bits.push(bit);
             }
         }
@@ -241,6 +267,7 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
                             show_icons: ShowIcons::Never,
                             embed_hyperlinks: EmbedHyperlinks::Off,
                             is_a_tty: self.options.is_a_tty,
+                            absolute: Absolute::Off,
                         };
 
                         let target_name = FileName {
@@ -249,11 +276,10 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
                             target: None,
                             link_style: LinkStyle::FullLinkPaths,
                             options: target_options,
-                            mounted_fs: None,
                             mount_style: MountStyle::JustDirectoryNames,
                         };
 
-                        for bit in target_name.escaped_file_name() {
+                        for bit in target_name.escaped_file_name(filename_style_override) {
                             bits.push(bit);
                         }
 
@@ -289,15 +315,15 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
             }
         }
 
-        if let (MountStyle::MountInfo, Some(mount_details)) =
-            (self.mount_style, self.mounted_fs.as_ref())
-        {
-            // This is a filesystem mounted on the directory, output its details
-            bits.push(Style::default().paint(" ["));
-            bits.push(Style::default().paint(mount_details.source.clone()));
-            bits.push(Style::default().paint(" ("));
-            bits.push(Style::default().paint(mount_details.fstype.clone()));
-            bits.push(Style::default().paint(")]"));
+        if self.mount_style == MountStyle::MountInfo {
+            if let Some(mount_details) = self.file.mount_point_info() {
+                // This is a filesystem mounted on the directory, output its details
+                bits.push(Style::default().paint(" ["));
+                bits.push(Style::default().paint(mount_details.source.clone()));
+                bits.push(Style::default().paint(" ("));
+                bits.push(Style::default().paint(mount_details.fstype.clone()));
+                bits.push(Style::default().paint(")]"));
+            }
         }
 
         bits.into()
@@ -372,13 +398,16 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
     ///
     /// So in that situation, those characters will be escaped and highlighted in
     /// a different colour.
-    fn escaped_file_name<'unused>(&self) -> Vec<ANSIString<'unused>> {
+    fn escaped_file_name<'unused>(
+        &self,
+        style_override: Option<Style>,
+    ) -> Vec<ANSIString<'unused>> {
         use percent_encoding::{utf8_percent_encode, CONTROLS};
 
         const HYPERLINK_START: &str = "\x1B]8;;";
         const HYPERLINK_END: &str = "\x1B\x5C";
 
-        let file_style = self.style();
+        let file_style = style_override.unwrap_or(self.style());
         let mut bits = Vec::new();
 
         let mut display_hyperlink = false;
@@ -403,7 +432,7 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
         }
 
         escape(
-            self.file.name.clone(),
+            self.display_name(),
             &mut bits,
             file_style,
             self.colours.control_char(),
@@ -419,6 +448,24 @@ impl<'a, 'dir, C: Colours> FileName<'a, 'dir, C> {
         bits
     }
 
+    /// Returns the string that should be displayed as the file's name.
+    fn display_name(&self) -> String {
+        match self.options.absolute {
+            Absolute::On => std::env::current_dir().ok().and_then(|p| {
+                path_clean::clean(p.join(&self.file.path))
+                    .to_str()
+                    .map(std::borrow::ToOwned::to_owned)
+            }),
+            Absolute::Follow => self
+                .file
+                .absolute_path()
+                .and_then(|p| p.to_str())
+                .map(std::borrow::ToOwned::to_owned),
+            Absolute::Off => None,
+        }
+        .unwrap_or(self.file.name.clone())
+    }
+
     /// Figures out which colour to paint the filename part of the output,
     /// depending on which “type” of file it appears to be — either from the
     /// class on the filesystem or from its name. (Or the broken link colour,
@@ -489,4 +536,6 @@ pub trait Colours: FiletypeColours {
     fn mount_point(&self) -> Style;
 
     fn colour_file(&self, file: &File<'_>) -> Style;
+
+    fn style_override(&self, file: &File<'_>) -> Option<FileNameStyle>;
 }
