@@ -230,10 +230,11 @@ impl Column {
 
 /// Formatting options for file sizes.
 #[allow(clippy::enum_variant_names)]
-#[derive(PartialEq, Eq, Debug, Copy, Clone)]
+#[derive(PartialEq, Eq, Debug, Default, Copy, Clone)]
 pub enum SizeFormat {
     /// Format the file size using **decimal** prefixes, such as “kilo”,
     /// “mega”, or “giga”.
+    #[default]
     DecimalBytes,
 
     /// Format the file size using **binary** prefixes, such as “kibi”,
@@ -262,12 +263,6 @@ pub enum GroupFormat {
     Smart,
 }
 
-impl Default for SizeFormat {
-    fn default() -> Self {
-        Self::DecimalBytes
-    }
-}
-
 /// The types of a file’s time fields. These three fields are standard
 /// across most (all?) operating systems.
 #[derive(PartialEq, Eq, Debug, Copy, Clone)]
@@ -308,20 +303,15 @@ impl TimeType {
 }
 
 /// How display file flags.
-#[derive(PartialEq, Eq, Debug, Copy, Clone)]
+#[derive(PartialEq, Eq, Debug, Default, Copy, Clone)]
 pub enum FlagsFormat {
     /// Display flags as comma seperated descriptions
+    #[default]
     Long,
     /// Display flags as single character abbreviations (Windows only)
     Short,
 }
 
-impl Default for FlagsFormat {
-    fn default() -> Self {
-        Self::Long
-    }
-}
-
 impl FlagsFormat {
     pub(crate) fn deduce<V: Vars>(vars: &V) -> FlagsFormat {
         vars.get(EZA_WINDOWS_ATTRIBUTES)
@@ -461,7 +451,7 @@ impl<'a> Table<'a> {
         let cells = self
             .columns
             .iter()
-            .map(|c| TextCell::paint_str(self.theme.ui.header, c.header()))
+            .map(|c| TextCell::paint_str(self.theme.ui.header.unwrap_or_default(), c.header()))
             .collect();
 
         Row { cells }
@@ -530,7 +520,7 @@ impl<'a> Table<'a> {
             #[cfg(unix)]
             Column::HardLinks => file.links().render(self.theme, &self.env.numeric),
             #[cfg(unix)]
-            Column::Inode => file.inode().render(self.theme.ui.inode),
+            Column::Inode => file.inode().render(self.theme.ui.inode.unwrap_or_default()),
             #[cfg(unix)]
             Column::Blocksize => {
                 file.blocksize()
@@ -551,22 +541,26 @@ impl<'a> Table<'a> {
             ),
             #[cfg(unix)]
             Column::SecurityContext => file.security_context().render(self.theme),
-            Column::FileFlags => file.flags().render(self.theme.ui.flags, self.flags_format),
+            Column::FileFlags => file
+                .flags()
+                .render(self.theme.ui.flags.unwrap_or_default(), self.flags_format),
             Column::GitStatus => self.git_status(file).render(self.theme),
             Column::SubdirGitRepo(status) => self.subdir_git_repo(file, status).render(self.theme),
             #[cfg(unix)]
-            Column::Octal => self.octal_permissions(file).render(self.theme.ui.octal),
+            Column::Octal => self
+                .octal_permissions(file)
+                .render(self.theme.ui.octal.unwrap_or_default()),
 
             Column::Timestamp(time_type) => time_type.get_corresponding_time(file).render(
                 if color_scale_info.is_some_and(|csi| csi.options.mode == ColorScaleMode::Gradient)
                 {
                     color_scale_info.unwrap().apply_time_gradient(
-                        self.theme.ui.date,
+                        self.theme.ui.date.unwrap_or_default(),
                         file,
                         time_type,
                     )
                 } else {
-                    self.theme.ui.date
+                    self.theme.ui.date.unwrap_or_default()
                 },
                 self.env.time_offset,
                 self.time_format.clone(),
